@@ -190,9 +190,12 @@ module Dnsruby
       elsif (arg.kind_of?Hash)
         arg.keys.each do |attr|
           begin
-            send(attr.to_s+"=", arg[attr])
-          rescue Exception
-            Dnsruby.log.error{"Argument #{attr} not valid\n"}
+            if ((attr.to_s == "src_address") && ((arg[attr] == nil) || (arg[attr] == "")))
+            else
+              send(attr.to_s+"=", arg[attr])
+            end
+          rescue Exception => e
+            Dnsruby.log.error{"PacketSender : Argument #{attr}, #{arg[attr]} not valid : #{e}\n"}
           end
           #        end
         end
@@ -321,10 +324,10 @@ module Dnsruby
       if (udp_packet_size < query_packet.length)
         if (@no_tcp)
           # Can't send the message - abort!
-              err=IOError.new("Can't send message - too big for UDP and no_tcp=true")
-              Dnsruby.log.error{"#{err}"}
-              st.push_exception_to_select(client_query_id, client_queue, err, nil)
-              return
+          err=IOError.new("Can't send message - too big for UDP and no_tcp=true")
+          Dnsruby.log.error{"#{err}"}
+          st.push_exception_to_select(client_query_id, client_queue, err, nil)
+          return
         end
         Dnsruby.log.debug{"Query packet length exceeds max UDP packet size - using TCP"}
         use_tcp = true
@@ -372,7 +375,10 @@ module Dnsruby
           runnextportloop = false
         rescue Exception => e
           if (socket!=nil)
-            socket.close
+            begin
+              socket.close
+            rescue Exception
+            end
           end
           # Try again if the error was EADDRINUSE and a random source port is used
           # Maybe try a max number of times?
@@ -409,7 +415,10 @@ module Dnsruby
         err=IOError.new("Send failed to #{@server}:#{@port} from #{@src_address}:#{src_port}, use_tcp=#{use_tcp}, exception : #{e}")
         Dnsruby.log.error{"#{err}"}
         st.push_exception_to_select(client_query_id, client_queue, err, nil)
-        socket.close
+        begin
+          socket.close
+        rescue Exception
+        end
         return
       end
       
