@@ -63,6 +63,8 @@ module Dnsruby
         unquoted = false
         seen_strings = false
         pos = 0
+        input.sub!(/^\s*\(\s*/, "")
+        input.sub!(/\s*\)\s*$/, "")
         input.each_char {|c|
           if (((c == "'") || (c == '"')) && (!in_escaped) && (!unquoted))
             if (!in_string)
