@@ -203,6 +203,13 @@ module Dnsruby
       # Note that a freestanding "@" is used to denote the current origin - we can simply replace that straight away
       # Remove the ( and )
       # Note that no domain name may be specified in the RR - in that case, last_name should be used. How do we tell? Tab or space at start of line.
+
+      # If we have text in the record, then ignore that in the parsing, and stick it on again at the end
+      stored_line = "";
+      if (line.index('"') != nil)
+          stored_line = line[line.index('"'), line.length];
+          line = line [0, line.index('"')]
+      end
       if ((line[0,1] == " ") || (line[0,1] == "\t"))
         line = @last_name + " " + line
       end
@@ -217,7 +224,7 @@ module Dnsruby
       split = line.split(' ') # split on whitespace
       name = split[0].strip
       if (name.index"\\")
-
+        
         ls =[]
         Name.create(name).labels.each {|el| ls.push(Name.decode(el.to_s))}
         new_name = ls.join('.')
@@ -324,14 +331,19 @@ module Dnsruby
         end
       end
 
-      line = line.split.join(' ').strip
+      line = line.strip
+
+      if (stored_line && stored_line != "")
+        line += " " + stored_line.strip
+      end
+
       # We need to fix up any non-absolute names in the RR
       # Some RRs have a single name, at the end of the string -
       #   to do these, we can just check the last character for "." and add the
       #   "." + origin string if necessary
       if ([Types::MX, Types::NS, Types::AFSDB, Types::NAPTR, Types::RT,
             Types::SRV, Types::CNAME, Types::MB, Types::MG, Types::MR,
-            Types::PTR].include?type_was)
+            Types::PTR, Types::DNAME].include?type_was)
         #        if (line[line.length-1, 1] != ".")
         if (!(/\.\z/ =~ line))
           line = line + "." + @origin.to_s + "."
@@ -359,14 +371,13 @@ module Dnsruby
         when Types::RP
           if (!parsed_rr.mailbox.absolute?)
             parsed_rr.mailbox = parsed_rr.mailbox.to_s + "." + @origin.to_s
-            if (!parsed_rr.txtdomain.absolute?)
-              parsed_rr.txtdomain = parsed_rr.txtdomain.to_s + "." + @origin.to_s
-            end
+          end
+          if (!parsed_rr.txtdomain.absolute?)
+            parsed_rr.txtdomain = parsed_rr.txtdomain.to_s + "." + @origin.to_s
           end
         end
         line = parsed_rr.to_s
       end
-
       if (do_prefix_hack)
         return line + "\n", type_string, @last_name
       end
